#include "config.h"

#include "ethernet_entry.h"


// DAC ISR
// A-channel, 1x, active
#define DAC_config_chan_A 0b0011000000000000
// B-channel, 1x, active
#define DAC_config_chan_B 0b1011000000000000

#include "jacktrip_packets.h"


void __ISR(_DMA_0_VECTOR, ipl2) start_send_packet_0(void){
    mDmaChnClrIntFlag(0);
    *((uint8_t * )&DCH0INT) = 0;
    outgoing_ptr = outgoing_packets;
    //g_playback_current = outgoing_packets[0].v_from_adc;
    //g_playback_end = g_playback_current+64;
}

void __ISR(_DMA_1_VECTOR, ipl2) start_send_packet_1(void){
    DmaChnEnable(0);
    mDmaChnClrIntFlag(1);
    *((uint8_t * )&DCH1INT) = 0;
    outgoing_ptr = outgoing_packets + 1;
    //g_playback_current = outgoing_packets[1].v_from_adc;
    //g_playback_end = g_playback_current+64;
}

void __ISR(_TIMER_3_VECTOR, ipl2) Timer3Handler(void)
{
    // you MUST clear the ISR flag
    mT3ClearIntFlag();
    if (g_playback_current)
        writeSampleToDAC();
}


void main(void){
    
    incoming_ptr = 0;
    outgoing_ptr = NULL;
    g_playback_current = NULL;
    g_playback_end = NULL;
    g_from_adc_buffer_head=0;
    
    {
    int i, ii;
    for (i=0; i<2; i++){
        outgoing_packets[i].header.TimeStamp = 0;
        outgoing_packets[i].header.SeqNumber = i;
        outgoing_packets[i].header.ConnectionMode = 0;
        outgoing_packets[i].header.BufferSize = 64;
        outgoing_packets[i].header.SamplingRate = 3;
        outgoing_packets[i].header.BitResolution = 16;
        outgoing_packets[i].header.NumChannels = 1;
        
        for (ii=0; ii<64; ii++){
            outgoing_packets[i].v_from_adc[ii][0] = 0;
            
        }
    }
    }
    
    //INTEnableSystemMultiVectoredInt();
    ANSELA = 0;
    ANSELB =0; //make sure analog is cleared
    
    INTEnableSystemMultiVectoredInt();
    
    
    
    //MIC Code
    
    // timer 3 setup for adc trigger  ==============================================
    // Set up timer3 on,  no interrupts, internal clock, prescalar 1, compare-value
    // This timer generates the time base for each ADC sample. 
    // works at ??? Hz
    #define sample_rate 48000
    // 40 MHz PB clock rate
    #define timer_match 40000000/sample_rate
    OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_1, 40000000/sample_rate);
    
    // set up the timer interrupt with a priority of 2
    ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_2);
    mT3ClearIntFlag(); // and clear the interrupt flag
    
    // control CS for DAC
    mPORTBSetPinsDigitalOut(BIT_9);
    mPORTBSetBits(BIT_9);
    // SCK2 is pin 26 
    // SDO2 (MOSI) is in PPS output group 2, could be connected to RB5 which is pin 14
    PPSOutput(2, RPB5, SDO2);
    // 16 bit transfer CKP=1 CKE=1
    // possibles SPI_OPEN_CKP_HIGH;   SPI_OPEN_SMP_END;  SPI_OPEN_CKE_REV
    // For any given peripherial, you will need to match these
    SpiChnOpen(SPI_CHANNEL2, SPI_OPEN_ON | SPI_OPEN_MODE16 | SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV , 2);
    
    
     #define DMAchan0 0
    #define DMAchan1 1
    DmaChnOpen(DMAchan0, 3, DMA_OPEN_DEFAULT);
	DmaChnOpen(DMAchan1, 2, DMA_OPEN_CHAIN_HI);
    /*
     * Additional explanation for what is going on here:
     * Read from &ADC1BUF0 which is 2 bytes large. The destination is nSamp*2 in size,
     * transferring 2 bytes per chunk.
     */
    //interleave(vin_not_interleaved[0], outgoing_packets[0].v_from_adc);
    
    DmaChnSetTxfer(DMAchan0, (void*)&ADC1BUF0, (void*)outgoing_packets[0].v_from_adc, 2, SAMPLES_PER_PACKET*2, 2); //64 16-bit fractionals
    DmaChnSetTxfer(DMAchan1, (void*)&ADC1BUF0, (void*)outgoing_packets[1].v_from_adc, 2, SAMPLES_PER_PACKET*2, 2); //64 16-bit fractionals
    
    mDmaChnIntEnable(0);
    mDmaChnIntEnable(1);
    
    mDmaChnSetIntPriority(0, 2, 0);
    mDmaChnSetIntPriority(1, 2, 0);
    
    //Chain DMAs
    //DmaChnSetTxfer(DMAchan1, (void*)&ADC1BUF1, (void*)(void*)vin_not_interleaved[0][1], 2, 128, 2); //512 16-bit integers
    DmaChnSetEventControl(DMAchan0, DMA_EV_START_IRQ(28)); // 28 is ADC done
    DmaChnSetEventControl(DMAchan1, DMA_EV_START_IRQ(28)); // 28 is ADC done
    //DmaChnSetEventControl(DMAchan1, DMA_EV_START_IRQ(28)); // Chain from first
    
    //Enable interrupt on block transfer complete 
    //DCH0INTbits.CHBCIE = 1;
    //DCH1INTbits.CHBCIE = 1;
    
    DmaChnSetEvEnableFlags(0, DMA_EV_BLOCK_DONE);
    DmaChnSetEvEnableFlags(1, DMA_EV_BLOCK_DONE);
    // ==============================================================================
    
    // configure and enable the ADC
    CloseADC10(); // ensure the ADC is off before setting the configuration

    // define setup parameters for OpenADC10
    // Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
    //ADC_FORMAT_SIGN_FRACT16
    
    #define PARAM1  ADC_FORMAT_SIGN_FRACT16 | ADC_CLK_TMR | ADC_AUTO_SAMPLING_ON //

    // define setup parameters for OpenADC10
    // ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off
    #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    //
    // Define setup parameters for OpenADC10
    // for a 40 MHz PB clock rate
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy should work at 40 MHz.
    // ADC_SAMPLE_TIME_6 seems to work with a source resistance < 1kohm
    // Tcy
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_6 | ADC_CONV_CLK_Tcy //ADC_SAMPLE_TIME_5| ADC_CONV_CLK_Tcy2

    // define setup parameters for OpenADC10
    // set AN11 and  as analog inputs
    #define PARAM4	ENABLE_AN0_ANA | ENABLE_AN1_ANA // pin 24

    // define setup parameters for OpenADC10
    // do not assign channels to scan
    #define PARAM5	SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15


    // use ground as neg ref for A | use AN11 for input A     
    // configure to sample AN11 
    SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF ); // configure to sample AN4 
    OpenADC10(PARAM1, PARAM2, PARAM3, PARAM4, PARAM5); // configure ADC using the parameters defined above

    EnableADC10(); // Enable the ADC
    ///////////////////////////////////////////////////////

     DmaChnEnable(0);
    
    //End MIC Code
    
    PPSOutput(3, RPB13, SDO1);
    PPSInput(2, SDI1, RPB8);
    
    
    
   
    
    //mPORTASetBits(BIT_0 );
    //LATAbits.LATA0 = 1;
    int i;
    
    ethernetMain();
    
    //mPORTAClearBits(BIT_0 );	//Clear bits to ensure light is off.
    //mPORTASetPinsDigitalOut(BIT_0);    //Set port as output
    
    while(1){
        for(i=0; i<4000000; i++){
            
        }
        mPORTAClearBits(BIT_0 );
        
        for(i=0; i<4000000; i++){
            
        }
        mPORTASetBits(BIT_0 );
        
    }
    
}